import type { Ref } from 'vue'
import { h } from 'vue'
import { Avatar } from 'ant-design-vue'
import { getRoleTree } from '@/api/system/role'
import type { BasicColumn, FormSchema } from '@/components/Table'
import { getAllTenants } from '@/api/system/tenant'
import { getDeptTree, lazyGetDeptList } from '@/api/system/dept'

export const columns: BasicColumn[] = [
  {
    title: '登陆账号',
    dataIndex: 'account',
    width: 180,
  },
  {
    title: '所属租户',
    dataIndex: 'tenantName',
    width: 100,
  },
  {
    title: '用户姓名',
    dataIndex: 'realName',
    width: 120,
    customRender({ record, value }) {
      return h('div', [
        h(
          Avatar,
          {
            src: /^https?/.test(record.avatar) ? record.avatar : undefined,
            size: 'small',
            style: { marginRight: '10px' },
          },
          () => value?.slice(0, 1),
        ),
        value,
      ])
    },
  },
  {
    title: '手机号码',
    dataIndex: 'mobile',
    width: 120,
  },
  {
    title: '用户邮箱',
    dataIndex: 'email',
    width: 120,
  },
  {
    title: '所属角色',
    dataIndex: 'roleName',
    width: 120,
  },
  {
    title: '所属部门',
    dataIndex: 'deptName',
    width: 120,
  },
]

export const searchFormSchema: FormSchema[] = [
  {
    label: '账号',
    field: 'account',
    component: 'Input',
    colProps: { span: 6 },
  },
  {
    label: '手机号码',
    field: 'mobile',
    component: 'InputNumber',
    componentProps: {
      controls: false,
      precision: 0,
    },
    colProps: { span: 6 },
  },
  {
    label: '所属部门',
    field: 'tenantId',
    component: 'ApiTreeSelect',
    componentProps: {
      async api() {
        try {
          const res = await lazyGetDeptList()
          return res.map(item => ({ ...item, isLeaf: !item.hasChildren }))
        }
        catch {
          return []
        }
      },
      async loadData(treeNode) {
        try {
          const res = await lazyGetDeptList({ parentId: treeNode.id })
          return res.map(item => ({ ...item, isLeaf: !item.hasChildren }))
        }
        catch {
          return []
        }
      },
      valueField: 'id',
      labelField: 'deptName',
    },
    colProps: { span: 6 },
  },
]

export function getFormSchema(isUpdate: Ref<boolean>): FormSchema[] {
  return [
    {
      field: 'id',
      show: false,
      component: 'Input',
    },
    {
      field: 'BaseInfo',
      label: '基本信息',
      component: 'Divider',
      componentProps: {
        plain: false,
      },
    },
    {
      label: '账号',
      field: 'account',
      required: true,
      component: 'Input',
      ifShow: () => !isUpdate.value,
    },
    {
      label: '密码',
      field: 'password',
      required: true,
      component: 'InputPassword',
      ifShow: () => !isUpdate.value,
    },
    {
      label: '确认密码',
      field: 'confirmPassword',
      required: true,
      component: 'InputPassword',
      ifShow: () => !isUpdate.value,
      dynamicRules: ({ values }) => {
        return [
          {
            required: true,
            validator: (_, value) => {
              if (value !== values.password)
                // eslint-disable-next-line prefer-promise-reject-errors
                return Promise.reject('两次输入的密码不一致!')

              return Promise.resolve()
            },
          },
        ]
      },
    },
    {
      label: '所属租户',
      field: 'tenantId',
      required: true,
      component: 'ApiSelect',
      defaultValue: undefined,
      dynamicDisabled: () => isUpdate.value,
      componentProps({ formActionType, formModel }) {
        return {
          api: getAllTenants,
          valueField: 'tenantId',
          labelField: 'tenantName',
          onChange(value?: string) {
            function updateRole(treeData: { id: string, title: string }[]) {
              if (!isUpdate)
                formModel.roleId && formActionType.setFieldsValue({ roleId: undefined })

              formActionType.updateSchema({
                field: 'roleId',
                componentProps: {
                  treeData,
                },
              })
            }
            function updateDept(treeData: { id: string, title: string }[]) {
              if (!isUpdate)
                formModel.deptId && formActionType.setFieldsValue({ deptId: undefined })

              formActionType.updateSchema({
                field: 'deptId',
                componentProps: {
                  treeData,
                },
              })
            }

            if (!value) {
              updateRole([])
              updateDept([])
              return
            }

            getRoleTree({ tenantId: value })
              .then((res) => {
                updateRole(res)
              })

            getDeptTree({ tenantId: value })
              .then((res) => {
                updateDept(res)
              })
          },
        }
      },
    },
    {
      label: '所属部门',
      field: 'deptId',
      required: true,
      component: 'TreeSelect',
      componentProps: {
        treeData: [],
        fieldNames: {
          label: 'title',
          value: 'id',
        },
      },
    },
    {
      label: '所属角色',
      field: 'roleId',
      required: true,
      component: 'TreeSelect',
      componentProps: {
        treeData: [],
        fieldNames: {
          label: 'title',
          value: 'id',
        },
      },
    },
    {
      field: 'UserInfo',
      label: '用户信息',
      component: 'Divider',
      componentProps: {
        plain: false,
      },
    },
    {
      label: '用户姓名',
      field: 'realName',
      component: 'Input',
      required: true,
    },
    {
      label: '用户头像',
      field: 'avatar',
      component: 'FileUpload',
      componentProps: {
        maxCount: 1,
        fileType: 'image',
      },
    },
    {
      label: '用户邮箱',
      field: 'email',
      component: 'Input',
      rules: [
        { type: 'email', message: '邮箱格式不正确' },
      ],
    },
    {
      label: '手机号码',
      field: 'mobile',
      component: 'InputNumber',
      componentProps: {
        controls: false,
        precision: 0,
      },
      rules: [
        { pattern: /^1[3-9]\d{9}$/, message: '手机号码格式不正确' },
      ],
    },
    {
      label: '用户性别',
      field: 'sex',
      component: 'Select',
      componentProps: {
        options: [
          {
            label: '男',
            value: 0,
          },
          {
            label: '女',
            value: 1,
          },
          {
            label: '未知',
            value: 2,
          },
        ],
      },
    },
    {
      label: '备注',
      field: 'remark',
      component: 'InputTextArea',
    },
  ]
}
